
// gkrell_sunlight.cc -- part of gkrell_sunlight
//
// This program is copyright (C) 2008 Mauro Persano, and is free
// software which is freely distributable under the terms of the
// GNU public license, included as the file COPYING in this
// distribution.  It is NOT public domain software, and any
// redistribution not permitted by the GNU General Public License is
// expressly forbidden without prior written permission from
// the author.

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>

#include "vector3.h"
#include "image.h"
#include "panic.h"
#include "sunlight_map.h"
#include "version.h"

extern "C" {
#include <gkrellm2/gkrellm.h>
}

static gchar CONFIG_NAME[] = "GKrellSunlight";
static gchar STYLE_NAME[] = "GKrellSunlight";

static const int UPDATE_INTERVAL = 30*1000; // update every 30 seconds

static GkrellmChart *chart;
static GkrellmChartconfig *chart_config;
static sunlight_map *map;

static void
create_plugin(GtkWidget *vbox, gint first_create);

static void
create_plugin_tab(GtkWidget *tab_vbox);

static void
apply_plugin();

static void
save_config(FILE *file);

static void
load_config(gchar *arg);

static GkrellmMonitor plugin_mon =
{
	CONFIG_NAME,        	// Name, for config tab.
	0,			// Id, 0 if a plugin 
	create_plugin,		// The create function 
	0,			// The update function 
	create_plugin_tab,	// The config tab create function 
	apply_plugin,		// Apply the config function 
	save_config,		// Save user config 
	load_config,		// Load user config 
	CONFIG_NAME,		// Config keyword 
	0,			// Undefined 2	
	0,			// Undefined 1	
	0,			// Undefined 0	
	MON_CPU,		// Insert plugin before this monitor
	0,			// Handle if a plugin, filled in by GKrellM 
	0			// path if a plugin, filled in by GKrellM   
};

static void
save_config(FILE *file)
{
}

static void
load_config(gchar *arg)
{
}

static void
apply_plugin()
{
}

static void
create_plugin_tab(GtkWidget *tab_vbox)
{
	GtkWidget *tabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

	char about_str[256];

	sprintf(about_str, 
	  "gkrell_sunlight %s\n"
	  "GKrellM Sunlight Plugin\n"
	  "\n"
	  "Copyright (c) 2008 Mauro Persano\n"
	  "m@fzort.org <-- e-mail me\n"
	  "\n"
	  "GPL v2",
	  VERSION);

	GtkWidget *label = gtk_label_new("About");
	GtkWidget *text = gtk_label_new(about_str);
	gtk_notebook_append_page(GTK_NOTEBOOK(tabs), text, label);
}

static gboolean
update_map(void *dummy)
{
	map->update(time(0));

	GdkEventExpose event;
	gint rv;

	gtk_signal_emit_by_name(GTK_OBJECT(chart->drawing_area),
	  "expose_event", &event, &rv);

	return TRUE;
}

static gint
chart_expose_event(GtkWidget *widget, GdkEventExpose *event)
{
	 gdk_draw_rgb_image(widget->window,
	  widget->style->fg_gc[GTK_STATE_NORMAL], 0, 0,
	  map->get_width(), map->get_height(),
	  GDK_RGB_DITHER_MAX,
	  reinterpret_cast<guchar *>(map->get_rgb_buffer()),
	  map->get_width()*3);

	return TRUE;
}

static void
create_plugin(GtkWidget *vbox, gint first_create)
{
	int chart_width = gkrellm_chart_width();

	if (first_create) {
		chart = gkrellm_chart_new0();
		map = new sunlight_map(chart_width);
	} else {
		map->set_width(chart_width);
	}

	map->update(time(0));

	gkrellm_chart_create(vbox, &plugin_mon, chart, &chart_config);
	gkrellm_set_chart_height(chart, map->get_height());

	if (first_create) {
		gtk_signal_connect(GTK_OBJECT(chart->drawing_area),
		  "expose_event", (GtkSignalFunc)chart_expose_event, 0);
		gtk_timeout_add(UPDATE_INTERVAL, update_map, 0);
	}
}

extern "C" GkrellmMonitor *
gkrellm_init_plugin()
{
	gkrellm_add_meter_style(&plugin_mon, STYLE_NAME);
	return &plugin_mon;
}
