
// image.h -- part of gkrell_sunlight
//
// This program is copyright (C) 2008 Mauro Persano, and is free
// software which is freely distributable under the terms of the
// GNU public license, included as the file COPYING in this
// distribution.  It is NOT public domain software, and any
// redistribution not permitted by the GNU General Public License is
// expressly forbidden without prior written permission from
// the author.

#ifndef IMAGE_H_
#define IMAGE_H_

#include <stdio.h>

struct rgb {
	union {
		struct {
			unsigned char r, g, b;
		};
		unsigned char arr[3];
	};

	rgb(unsigned char r_ = 0, unsigned char g_ = 0, unsigned char b_ = 0)
	  : r(r_), g(g_), b(b_)
	{ }
};

struct image
{
  public:
	image(int width_, int height_)
	  : width(width_), height(height_), data(new rgb[width*height])
	{ }

	virtual ~image()
	{ delete[] data; }

#ifdef USE_PNG
	static image *from_png(const char *file_name);
#endif
	static image *from_xpm_data(char **xpm_data);

	int get_width() const { return width; }
	int get_height() const { return height; }

	rgb& pixel_at(int r, int c) { return data[r*width + c]; }
	rgb pixel_at(int r, int c) const { return data[r*width + c]; }

	void to_ppm(FILE *fp) const;

  private:
	int width;
	int height;
	rgb *data;

	// copy ctor and assignment disabled
	image(const image& image);
	image& operator=(const image& image);

	friend class sunlight_map;
};

#endif // IMAGE_H_
