
// vector3.h -- part of gkrell_sunlight
//
// This program is copyright (C) 2008 Mauro Persano, and is free
// software which is freely distributable under the terms of the
// GNU public license, included as the file COPYING in this
// distribution.  It is NOT public domain software, and any
// redistribution not permitted by the GNU General Public License is
// expressly forbidden without prior written permission from
// the author.

#ifndef VECTOR3_H_
#define VECTOR3_H_

#include <cmath>

struct vector3 {
	float x, y, z;

	vector3(float x_ = 0, float y_ = 0, float z_ = 0)
	  : x(x_), y(y_), z(z_)
	{ }

	float
	dot_product(const vector3& v) const
	{ return x*v.x + y*v.y + z*v.z; }

	float
	length_squared() const
	{ return dot_product(*this); }

	float
	length() const
	{ return std::sqrt(length_squared()); }

	vector3
	operator*(float s) const
	{ return vector3(x*s, y*s, z*s); }

	vector3
	normalize() const
	{
		float l = length();
		return l == 0.f ? vector3(1, 0, 0) : *this*(1/l);
	}
};

#endif // VECTOR3_H_
